<?php
namespace AIOSEO\Plugin\Pro\Api;

use AIOSEO\Plugin\Common\Api as CommonApi;

/**
 * Route class for the API.
 *
 * @since 4.0.0
 */
class PostsTerms extends CommonApi\PostsTerms {
	/**
	 * Update post settings.
	 *
	 * @since 4.0.0
	 *
	 * @param  \WP_REST_Request  $request The REST Request
	 * @return \WP_REST_Response          The response.
	 */
	public static function updatePosts( $request ) {
		$body               = $request->get_json_params();
		$postId             = ! empty( $body['id'] ) ? intval( $body['id'] ) : null;
		$context            = ! empty( $body['context'] ) ? sanitize_text_field( $body['context'] ) : 'post';
		$theId              = ( 'term' === $context ) ? 'term_id' : 'post_id';
		$model              = ( 'post' === $context ) ? 'AIOSEO\\Plugin\\Common\\Models\\Post' : 'AIOSEO\\Plugin\\Pro\\Models\\Term';
		$title              = ! empty( $body['title'] ) ? sanitize_text_field( $body['title'] ) : null;
		$description        = ! empty( $body['description'] ) ? sanitize_text_field( $body['description'] ) : null;
		$keywords           = ! empty( $body['keywords'] ) ? sanitize_text_field( $body['keywords'] ) : null;
		$ogTitle            = ! empty( $body['og_title'] ) ? sanitize_text_field( $body['og_title'] ) : null;
		$ogDescription      = ! empty( $body['og_description'] ) ? sanitize_text_field( $body['og_description'] ) : null;
		$ogArticleSection   = ! empty( $body['og_article_section'] ) ? sanitize_text_field( $body['og_article_section'] ) : null;
		$ogArticleTags      = ! empty( $body['og_article_tags'] ) ? sanitize_text_field( $body['og_article_tags'] ) : null;
		$twitterTitle       = ! empty( $body['twitter_title'] ) ? sanitize_text_field( $body['twitter_title'] ) : null;
		$twitterDescription = ! empty( $body['twitter_description'] ) ? sanitize_text_field( $body['twitter_description'] ) : null;

		if ( ! $postId ) {
			return new \WP_REST_Response( [
				'success' => false,
				'message' => __( 'Post ID is missing.', 'all-in-one-seo-pack' )
			], 400 );
		}

		$thePost = aioseo()->db
			->start( 'aioseo_' . $context . 's' )
			->where( $theId, $postId )
			->run()
			->model( $model );

		$thePost->{ $theId }                  = $postId;
		$thePost->priority                    = ! empty( $body['priority'] ) ? sanitize_text_field( $body['priority'] ) : null;
		$thePost->frequency                   = ! empty( $body['frequency'] ) ? sanitize_text_field( $body['frequency'] ) : null;
		$thePost->title                       = $title;
		$thePost->description                 = $description;
		$thePost->keywords                    = $keywords;
		$thePost->keyphrases                  = ! empty( $body['keyphrases'] ) ? wp_json_encode( $body['keyphrases'] ) : null;
		$thePost->page_analysis               = ! empty( $body['page_analysis'] ) ? wp_json_encode( $body['page_analysis'] ) : null;
		$thePost->seo_score                   = ! empty( $body['seo_score'] ) ? sanitize_text_field( $body['seo_score'] ) : 0;
		$thePost->robots_enabled              = ! empty( $body['robots_enabled'] ) ? sanitize_text_field( $body['robots_enabled'] ) : 0;
		$thePost->canonical_url               = ! empty( $body['canonicalUrl'] ) ? sanitize_text_field( $body['canonicalUrl'] ) : null;
		$thePost->pillar_content              = isset( $body['pillar_content'] ) ? rest_sanitize_boolean( $body['pillar_content'] ) : 0;
		$thePost->robots_default              = isset( $body['default'] ) ? rest_sanitize_boolean( $body['default'] ) : 1;
		$thePost->robots_noindex              = isset( $body['noindex'] ) ? rest_sanitize_boolean( $body['noindex'] ) : 0;
		$thePost->robots_nofollow             = isset( $body['nofollow'] ) ? rest_sanitize_boolean( $body['nofollow'] ) : 0;
		$thePost->robots_noarchive            = isset( $body['noarchive'] ) ? rest_sanitize_boolean( $body['noarchive'] ) : 0;
		$thePost->robots_notranslate          = isset( $body['notranslate'] ) ? rest_sanitize_boolean( $body['notranslate'] ) : 0;
		$thePost->robots_noimageindex         = isset( $body['noimageindex'] ) ? rest_sanitize_boolean( $body['noimageindex'] ) : 0;
		$thePost->robots_nosnippet            = isset( $body['nosnippet'] ) ? rest_sanitize_boolean( $body['nosnippet'] ) : 0;
		$thePost->robots_noodp                = isset( $body['noodp'] ) ? rest_sanitize_boolean( $body['noodp'] ) : 0;
		$thePost->robots_max_snippet          = ! empty( $body['maxSnippet'] ) ? sanitize_text_field( $body['maxSnippet'] ) : 0;
		$thePost->robots_max_videopreview     = ! empty( $body['maxVideoPreview'] ) ? (int) sanitize_text_field( $body['maxVideoPreview'] ) : 0;
		$thePost->robots_max_imagepreview     = ! empty( $body['maxImagePreview'] ) ? sanitize_text_field( $body['maxImagePreview'] ) : 'none';
		$thePost->og_object_type              = ! empty( $body['og_object_type'] ) ? sanitize_text_field( $body['og_object_type'] ) : 'default';
		$thePost->og_title                    = $ogTitle;
		$thePost->og_description              = $ogDescription;
		$thePost->og_image_custom_url         = ! empty( $body['og_image_custom_url'] ) ? sanitize_text_field( $body['og_image_custom_url'] ) : null;
		$thePost->og_image_custom_fields      = ! empty( $body['og_image_custom_fields'] ) ? sanitize_text_field( $body['og_image_custom_fields'] ) : null;
		$thePost->og_image_type               = ! empty( $body['og_image_type'] ) ? sanitize_text_field( $body['og_image_type'] ) : 'default';
		$thePost->og_video                    = ! empty( $body['og_video'] ) ? sanitize_text_field( $body['og_video'] ) : '';
		$thePost->og_article_section          = $ogArticleSection;
		$thePost->og_article_tags             = $ogArticleTags;
		$thePost->twitter_use_og              = isset( $body['twitter_use_og'] ) ? rest_sanitize_boolean( $body['twitter_use_og'] ) : 0;
		$thePost->twitter_card                = ! empty( $body['twitter_card'] ) ? sanitize_text_field( $body['twitter_card'] ) : 'default';
		$thePost->twitter_image_custom_url    = ! empty( $body['twitter_image_custom_url'] ) ? sanitize_text_field( $body['twitter_image_custom_url'] ) : null;
		$thePost->twitter_image_custom_fields = ! empty( $body['twitter_image_custom_fields'] ) ? sanitize_text_field( $body['twitter_image_custom_fields'] ) : null;
		$thePost->twitter_image_type          = ! empty( $body['twitter_image_type'] ) ? sanitize_text_field( $body['twitter_image_type'] ) : 'default';
		$thePost->twitter_title               = $twitterTitle;
		$thePost->twitter_description         = $twitterDescription;
		$thePost->schema_type                 = ! empty( $body['schema_type'] ) ? sanitize_text_field( $body['schema_type'] ) : 'none';
		$thePost->schema_type_options         = ! empty( $body['schema_type_options'] ) ? wp_json_encode( $body['schema_type_options'] ) : CommonModels\Post::getDefaultSchemaOptions();
		$thePost->tabs                        = ! empty( $body['tabs'] ) ? wp_json_encode( $body['tabs'] ) : CommonModels\Post::getDefaultTabsOptions();
		$thePost->local_seo                   = ! empty( $body['local_seo'] ) ? wp_json_encode( $body['local_seo'] ) : null;
		$thePost->updated                     = gmdate( 'Y-m-d H:i:s' );

		if ( ! $thePost->exists() ) {
			$thePost->created = gmdate( 'Y-m-d H:i:s' );
		}
		$thePost->save();
		$thePost->reset();

		$lastError = aioseo()->db->lastError();
		if ( ! empty( $lastError ) ) {
			return new \WP_REST_Response( [
				'success' => false,
				'message' => 'Failed update query: ' . $lastError
			], 401 );
		}

		// Update the post/term meta as well for localization.
		$keywords = json_decode( $keywords ) ? json_decode( $keywords ) : [];
		foreach ( $keywords as $k => $keyword ) {
			$keywords[ $k ] = $keyword->value;
		}
		$keywords = implode( ',', $keywords );

		$ogArticleTags = json_decode( $ogArticleTags ) ? json_decode( $ogArticleTags ) : [];
		foreach ( $ogArticleTags as $k => $tag ) {
			$ogArticleTags[ $k ] = $tag->value;
		}
		$ogArticleTags = implode( ',', $ogArticleTags );

		if ( 'post' === $context ) {
			update_post_meta( $postId, '_aioseo_title', $title );
			update_post_meta( $postId, '_aioseo_description', $description );
			update_post_meta( $postId, '_aioseo_keywords', $keywords );
			update_post_meta( $postId, '_aioseo_og_title', $ogTitle );
			update_post_meta( $postId, '_aioseo_og_description', $ogDescription );
			update_post_meta( $postId, '_aioseo_og_article_section', $ogArticleSection );
			update_post_meta( $postId, '_aioseo_og_article_tags', $ogArticleTags );
			update_post_meta( $postId, '_aioseo_twitter_title', $twitterTitle );
			update_post_meta( $postId, '_aioseo_twitter_description', $twitterDescription );
		} elseif ( 'term' === $context ) {
			update_term_meta( $postId, '_aioseo_title', $title );
			update_term_meta( $postId, '_aioseo_description', $description );
			update_term_meta( $postId, '_aioseo_keywords', $keywords );
			update_term_meta( $postId, '_aioseo_og_title', $ogTitle );
			update_term_meta( $postId, '_aioseo_og_description', $ogDescription );
			update_term_meta( $postId, '_aioseo_og_article_section', $ogArticleSection );
			update_term_meta( $postId, '_aioseo_og_article_tags', $ogArticleTags );
			update_term_meta( $postId, '_aioseo_twitter_title', $twitterTitle );
			update_term_meta( $postId, '_aioseo_twitter_description', $twitterDescription );
		}

		return new \WP_REST_Response( [
			'success' => true,
			'posts'   => $postId
		], 200 );
	}

	/**
	 * Update term settings from Term screen.
	 *
	 * @since 4.0.0
	 *
	 * @param  \WP_REST_Request  $request The REST Request
	 * @return \WP_REST_Response          The response.
	 */
	public static function updateTermFromScreen( $request ) {
		$body   = $request->get_json_params();
		$termId = ! empty( $body['termId'] ) ? intval( $body['termId'] ) : null;

		if ( ! $termId ) {
			return new \WP_REST_Response( [
				'success' => false,
				'message' => 'Term ID is missing.'
			], 400 );
		}

		$theTerm = aioseo()->db
			->start( 'aioseo_terms' )
			->where( 'term_id', $termId )
			->run()
			->model( 'AIOSEO\\Plugin\\Pro\\Models\\Term' );

		if ( $theTerm->exists() ) {
			$theTerm->title       = ! empty( $body['title'] ) ? sanitize_text_field( $body['title'] ) : '';
			$theTerm->description = ! empty( $body['description'] ) ? sanitize_text_field( $body['description'] ) : '';
			$theTerm->updated     = gmdate( 'Y-m-d H:i:s' );
		} else {
			$theTerm->term_id     = $termId;
			$theTerm->title       = ! empty( $body['title'] ) ? sanitize_text_field( $body['title'] ) : '';
			$theTerm->description = ! empty( $body['description'] ) ? sanitize_text_field( $body['description'] ) : '';
			$theTerm->created     = gmdate( 'Y-m-d H:i:s' );
			$theTerm->updated     = gmdate( 'Y-m-d H:i:s' );
		}
		$theTerm->save();

		$lastError = aioseo()->db->lastError();
		if ( ! empty( $lastError ) ) {
			return new \WP_REST_Response( [
				'success' => false,
				'message' => 'Failed update query: ' . $lastError
			], 401 );
		}

		return new \WP_REST_Response( [
			'success' => true,
			'terms'   => $termId
		], 200 );
	}
}